#!/usr/bin/env python3
"""Analyze installation steps to document subprocess commands."""

import json
from pathlib import Path
from typing import Dict, List
import sys

# Add utils to path before other imports
sys.path.insert(0, str(Path(__file__).parent.parent))

from utils import BaseCLI
from utils.shared_console import get_shared_console
from installer.steps import get_installation_steps


class AnalyzeInstallStepsCLI(BaseCLI):
    """Analyze installation steps to document subprocess commands."""
    
    def __init__(self):
        super().__init__(
            name="analyze-install-steps",
            help_text="Analyze installation steps and document subprocess commands",
            require_config=False
        )
        self.console = get_shared_console()
        self.setup_commands()
    
    def setup_commands(self):
        """Set up CLI commands."""
        
        @self.app.callback(invoke_without_command=True)
        def analyze():
            """Analyze all installation steps."""
            self.analyze_steps()
    
    def analyze_steps(self):
        """Analyze installation steps and save subprocess command mappings."""
        self.console.print("[bold]Analyzing installation steps...[/bold]\n")
        
        # Get all steps
        steps = get_installation_steps(skip_frontend=False)
        
        # Create mapping of commands to subprocess calls
        command_mapping = {}
        
        # Hardcoded mappings based on implementation knowledge
        # This would ideally be generated by analyzing the actual command implementations
        command_mapping.update({
            "docr setup": [
                "mkdir -p ~/.docr",
                "(prompts for project root)",
                "(prompts for developer initials)",
                "(writes ~/docr.toml)"
            ],
            "docr config backend --all": [
                "(reads ~/docr.toml)",
                "(generates samconfig.toml for each component)",
                "(no subprocess calls - pure Python)"
            ],
            "docr config backend oidc-manager": [
                "(reads ~/docr.toml)",
                "(generates samconfig.toml)",
                "(creates config/config.sandbox with env vars)"
            ],
            "docr config backend legislative-review": [
                "(reads ~/docr.toml)",
                "(generates samconfig.toml)",
                "(creates config/config.sandbox with API URLs)"
            ],
            "docr config frontend --app legislative-review": [
                "(reads ~/docr.toml)",
                "(reads OIDC client ID from DynamoDB)",
                "(creates .env.production with React env vars)"
            ],
            "docr config frontend --app oidc-manager": [
                "(reads ~/docr.toml)",
                "(reads OIDC client ID from DynamoDB)",
                "(creates .env.production with React env vars)"
            ],
            "docr refresh": [
                "aws cloudformation describe-stacks (parallel for all stacks)",
                "(parses CloudFormation outputs)",
                "(updates ~/docr.toml with latest values)"
            ],
            "docr deploy backend --app oidc-manager": [
                "docker build -t <image> .",
                "aws ecr get-login-password",
                "docker login <ecr-registry>",
                "docker tag <image> <ecr-uri>",
                "docker push <ecr-uri>",
                "sam deploy --image-uri <ecr-uri>"
            ],
            "docr deploy frontend --app legislative-review --full": [
                "npm install",
                "npm run build",
                "sam build",
                "sam deploy --no-confirm-changeset"
            ],
            "docr deploy frontend --app legislative-review": [
                "npm run build",
                "aws s3 sync ./build s3://<bucket>",
                "aws cloudfront create-invalidation --distribution-id <id>"
            ],
            "docr deploy frontend --app oidc-manager --full": [
                "npm install",
                "npm run build",
                "sam build",
                "sam deploy --no-confirm-changeset"
            ],
            "docr deploy frontend --app oidc-manager": [
                "npm run build",
                "aws s3 sync ./build s3://<bucket>",
                "aws cloudfront create-invalidation --distribution-id <id>"
            ],
            "docr bootstrap all --app legislative-review": [
                "(reads bootstrap/*.yml files)",
                "(makes API calls to each component)",
                "(POST requests with YAML data)"
            ],
            "docr oidc register --app legislative-review": [
                "cd oidc-oidcauthorizer-serverless",
                "npm run register-client",
                "(node script creates DynamoDB entry)"
            ],
            "docr oidc register --app oidc-manager": [
                "cd oidc-oidcauthorizer-serverless",
                "npm run register-client",
                "(node script creates DynamoDB entry)"
            ],
            "docr jobs sync --session 2025RS --app legislative-review": [
                "(API call to trigger Lambda)",
                "(Lambda fetches bills from external API)",
                "(Processes with OpenAI)",
                "(Stores in DynamoDB)"
            ],
            "docr jobs monitor --app legislative-review": [
                "(API call to trigger Lambda)",
                "(Lambda analyzes bills for impacts)",
                "(Updates DynamoDB records)"
            ],
            "docr credstore add-from-ssm openai": [
                "aws ssm get-parameter --name /docreview/openai/api-key",
                "(DynamoDB put-item to credential store)"
            ],
            "docr doctor all": [
                "aws cloudformation describe-stacks",
                "(validates config files)",
                "(tests API connectivity)",
                "(checks OIDC client registrations)"
            ],
            "docr show-urls": [
                "docr refresh (if needed)",
                "(reads ~/docr.toml)",
                "(displays frontend URLs)"
            ],
            "sam build": [
                "docker run --rm lambci/lambda:build-python3.9",
                "pip install -r requirements.txt",
                "(packages Lambda deployment)"
            ],
            "sam deploy": [
                "aws cloudformation package",
                "aws cloudformation deploy",
                "aws cloudformation wait stack-create-complete"
            ],
            "npm install": [
                "(reads package.json)",
                "(downloads dependencies to node_modules)",
                "(updates package-lock.json)"
            ],
            "aws cloudformation list-stacks": [
                "(AWS API call)",
                "(returns stack information)"
            ],
            "sleep": [
                "(OS sleep call)"
            ]
        })
        
        # Save mapping to file
        output_file = Path(__file__).parent.parent / "installer" / "subprocess_commands.json"
        with open(output_file, "w") as f:
            json.dump(command_mapping, f, indent=2)
        
        self.console.print(f"[green]✓ Saved subprocess command mappings to {output_file}[/green]")
        
        # Display summary
        self.console.print(f"\nAnalyzed {len(steps)} installation steps")
        self.console.print(f"Documented {len(command_mapping)} unique commands")


def main():
    """Main entry point."""
    cli = AnalyzeInstallStepsCLI()
    cli.run()


if __name__ == "__main__":
    main()